
#include "data.hh"
#include "language.hh"
#include "const_string.hh"

void WordList_Base::attach(const SC_Language &l) {
  if (lang_ && const_string(l.name()) != lang_->name())
    throw ae_mismatched_lang(l.name(), this);
  if (!lang_) lang_ = new SC_Language(l);
  ++attach_count_;
}

void WordList_Base::detach() {
  --attach_count_;
}

WordList_Base::WordList_Base()
  : lang_(0,false), attach_count_(0) {}

WordList_Base::~WordList_Base() {
}

bool WordList_Base::is_attached() const {
  return attach_count_;
}

const char * WordList_Base::lang_name() const {
  return lang_->name();
}

void WordList_Base::check_lang(const string &l) {
  if (l != lang_->name()) 
    throw ae_mismatched_lang(l, this);
}

void WordList_Base::set_check_lang (const string & l, 
				    const AspellConfig * config)
{
  if (!is_attached()) {
    lang_ = new SC_Language(l,config);
    if (!lang_)
      throw ae_unknown_lang(l, this);
  } else {
    if (l != lang_->name()) 
      throw ae_mismatched_lang(l, this);
  }
}




