
#include <strstream>
#include <fstream>

#include "string_map.hh"

//
// Utility Functions
//

static string trim_comments(const string & data) {
  string temp;
  string::const_iterator i = data.begin();
  string::const_iterator end = data.end();
  while (i != end && *i != '#') {temp += *i; ++i;}
  return temp;
}

static string trim_space(const string &s) {
  string::size_type beg = s.find_first_not_of(" \t");
  string::size_type end = s.find_last_not_of(" \t");
  if (beg == string::npos)
    return string();
  else
    return s.substr(beg,end-beg+1);
}

static bool getdata_pair(istream & in, string & key, string & data, char d) {
  string temp;
  if (!getline(in,temp,d)) return false;
  temp = trim_comments(temp);
  istrstream s(temp.c_str());
  if (!(s >> key)) return getdata_pair(in,key,data,d);
  data = "";
  do {
    if (!getline(s,temp)) temp = "";
    data += temp + " ";
  } while (temp[temp.size()-1]=='\\');
  data = trim_space(data);
  return true;
}

//
// StringMap
//

StringMap::StringMap() {}

StringMap::~StringMap() {}

StringMap::const_iterator StringMap::begin() const {
  return data.begin();
}

StringMap::const_iterator StringMap::end() const {
  return data.end();
}

void StringMap::merge(const StringMap & other) {
  const_iterator i   = other.begin();
  const_iterator end = other.end();
  while (i != end) {
    replace(i->first, i->second);
    ++i;
  }
}

bool StringMap::read_in_stream(istream & in, char delim) {
 string key,value;
  while (getdata_pair(in, key, value, delim)) {
    replace(key, value);
  }
  return true;
}

bool StringMap::read_in_file(const string & file) {
  ifstream in(file.c_str());
  if (in.rdstate() & ios::badbit) return false;
  read_in_stream(in);
  return true;
}

bool StringMap::read_in_string(const string & str) {
  istrstream in(str.c_str());
  read_in_stream(in, ';');
  return true;
}

bool StringMap::write_to_stream(ostream & out) const {
  const_iterator i = begin();
  const_iterator e = end();
  while ( i != e ) {
    out << i->first << '\t' << i->second << endl;
    ++i;
  }
  return true;
}

bool StringMap::write_to_file(const string & file) const {
  ofstream out(file.c_str());
  if (out.rdstate() & ios::badbit) return false;
  write_to_stream(out);
  return true;
}

const string & StringMap::lookup (const string & key) const {
  static const string empty_string = "";
  Data::const_iterator i = data.find(key);
  if (i == data.end()) 
    return empty_string;
  else 
    return i->second;
}

bool StringMap::have(const string & key) const {
  return data.count(key);
}

void StringMap::insert(const string & key, const string & value) {
  data.insert(Data::value_type(key,value));
}

void StringMap::replace(const string & key, const string & value) {
  data[key] = value;
}

void StringMap::remove(const string & key) {
  data.erase(key);
}

